@@ -67,6 +67,8 @@ kuryr_k8s_opts = [
                     "Pods"),
     cfg.BoolOpt("test_udp_services", default=False,
                 help="Whether or not service UDP tests will be running"),
+    cfg.BoolOpt("test_sctp_services", default=False,
+                help="Whether or not service SCTP tests will be running"),
     cfg.BoolOpt("multi_worker_setup", default=False, help="Whether or not we "
                 "have a multi-worker setup"),
     cfg.BoolOpt("cloud_provider", default=False, help="Whether or not a "
@@ -94,4 +96,7 @@ kuryr_k8s_opts = [
     cfg.BoolOpt("test_services_without_selector", default=False,
                 help="Whether or not service without selector tests will be "
                 "running"),
+    cfg.BoolOpt("test_configurable_listener_timeouts", default=False,
+                help="Whether or not listener timeout values are "
+                "configurable"),
 ]
