@@ -22,6 +22,7 @@ import time
 
 from oslo_log import log as logging
 
+import netaddr
 import requests
 
 import kubernetes
@@ -425,6 +426,18 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                 namespace=namespace,
                 body=delete_options)
 
+    @classmethod
+    def update_service(cls, service_name, annotation, namespace="default"):
+        api = cls.k8s_client.CoreV1Api()
+        service = api.read_namespaced_service(service_name, namespace)
+        service.metadata = cls.k8s_client.V1ObjectMeta(
+            name=service_name, annotations=annotation)
+
+        service_obj = api.patch_namespaced_service(body=service,
+                                                   name=service_name,
+                                                   namespace=namespace)
+        return service_obj
+
     @classmethod
     def get_service_ip(
             cls, service_name, spec_type="ClusterIP", namespace="default"):
@@ -513,6 +526,30 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         msg = "Timed out waiting for lb crd status %s" % service_name
         raise lib_exc.TimeoutException(msg)
 
+    @classmethod
+    def get_listener_timeout_on_crd(cls, service_name, namespace):
+        api = cls.k8s_client.CoreV1Api()
+        service = api.read_namespaced_service(service_name, namespace)
+
+        if service.metadata.annotations:
+            try:
+                klb_crd = cls.get_kuryr_loadbalancer_crds(
+                    service_name, namespace)
+            except kubernetes.client.rest.ApiException:
+                return None, None
+            klb_status = klb_crd.get('status')
+            if klb_status and klb_status.get('listeners'):
+                try:
+                    for l in klb_status.get('listeners', []):
+                        timeout_cli = l.get('timeout_client_data')
+                        timeout_mb = l.get('timeout_member_data')
+
+                    return timeout_cli, timeout_mb
+                except KeyError:
+                    LOG.info("Waiting till LB's timeout appears in CRD")
+                    return None, None
+        return None, None
+
     @classmethod
     def wait_kuryr_annotation(cls, group, version, plural, annotation,
                               timeout_period, name, namespace='default'):
@@ -628,31 +665,58 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                                       protocol="TCP", port=80,
                                       target_port=8080, label=None,
                                       namespace="default", get_ip=True,
-                                      service_name=None, cleanup=True):
+                                      service_name=None, cleanup=True,
+                                      save=True):
+        """Precreate resources for service test (booting amphoras takes time).
+
+        :param pod_num: Number of pods
+        :param spec_type: Service's spec.type
+        :param protocol: Listener protocol
+        :param port: Listener port
+        :param target_port: Port on members
+        :param label: Label to use on pods as {'app': label}
+        :param namespace: K8s namespace to use
+        :param get_ip: Should it save service_ip on the class?
+        :param service_name: Name of the Service
+        :param cleanup: Should it add service cleanup?
+        :param save: Should it save anything on the class? Useful for running
+                     in a single test.
+        :return: Tuple with (<name of service>, <list of service pod names>)
+        """
 
         label = label or data_utils.rand_name('kuryr-app')
+        svc_pods = []
         for i in range(pod_num):
-            pod_name, pod = cls.create_pod(
-                labels={"app": label}, namespace=namespace)
+            if protocol == "SCTP":
+                pod_name, pod = cls.create_pod(
+                    labels={"app": label}, image='quay.io/kuryr/sctp-demo',
+                    namespace=namespace)
+            else:
+                pod_name, pod = cls.create_pod(
+                    labels={"app": label}, namespace=namespace)
+            svc_pods.append(pod_name)
             if cleanup:
                 cls.addClassResourceCleanup(cls.delete_pod, pod_name,
                                             namespace=namespace)
-        cls.pod_num = pod_num
+        if save:
+            cls.pod_num = pod_num
         service_name, service_obj = cls.create_service(
             pod_label=pod.metadata.labels, spec_type=spec_type,
             protocol=protocol, port=port, target_port=target_port,
             namespace=namespace, service_name=service_name)
         if get_ip:
-            cls.service_ip = cls.get_service_ip(
-                service_name, spec_type=spec_type, namespace=namespace)
+            if save:
+                cls.service_name = service_name
+                cls.service_ip = cls.get_service_ip(
+                    service_name, spec_type=spec_type, namespace=namespace)
             # This is already waiting for endpoint annotations to be made by
             # Kuryr
             cls.verify_lbaas_endpoints_configured(service_name, pod_num,
                                                   namespace)
-            cls.service_name = service_name
         if cleanup:
             cls.addClassResourceCleanup(cls.delete_service, service_name,
                                         namespace=namespace)
+        return service_name, svc_pods
 
     @classmethod
     def service_without_selector_base(cls, pod_num=2,
@@ -724,12 +788,14 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         return False
 
     @classmethod
-    def create_namespace(cls, name=None, wait_for_crd=True,
+    def create_namespace(cls, name=None, labels=None,
+                         wait_for_crd=True,
                          timeout_period=consts.NS_TIMEOUT):
         if not name:
             name = data_utils.rand_name(prefix='kuryr-namespace')
         namespace = cls.k8s_client.V1Namespace()
-        namespace.metadata = cls.k8s_client.V1ObjectMeta(name=name)
+        namespace.metadata = cls.k8s_client.V1ObjectMeta(name=name,
+                                                         labels=labels)
         namespace_obj = cls.k8s_client.CoreV1Api().create_namespace(
             body=namespace)
 
@@ -953,6 +1019,20 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                 return
             return stdout
 
+        def req_sctp():
+            cmd = "python3 sctp_client.py {} {}".format(
+                server_ip, server_port)
+            pod_cmd = ["/bin/sh", "-c", cmd]
+            stdout, stderr = self.exec_command_in_pod(pod, pod_cmd,
+                                                      namespace=namespace_name,
+                                                      stderr=True)
+            if stderr:
+                LOG.error('Failed to reach service at {}:{} '
+                          'Err: {}'.format(server_ip, server_port, stderr))
+                time.sleep(10)
+                return
+            return stdout
+
         def pred(tester, responses):
             if protocol == 'TCP':
                 unique_resps = set(resp for resp in responses if resp)
@@ -969,6 +1049,10 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             self.assertIsNotNone(server_port, "server_port must be "
                                               "provided for UDP protocol")
             req = req_udp
+        elif protocol == "SCTP":
+            self.assertIsNotNone(server_port, "server_port must be "
+                                              "provided for SCTP protocol")
+            req = req_sctp
         else:
             LOG.info("Unsupported protocol %s, returning", protocol)
             return
@@ -1064,7 +1148,13 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
     def create_vm_for_connectivity_test(self):
         keypair = self.create_keypair()
-        sec_grp = self._create_security_group()
+        # NOTE(maysams): we need to support older versions of Tempest.
+        # So, let's try calling the non-private method and if fails
+        # we default to the private one.
+        try:
+            sec_grp = self._create_security_group()
+        except AttributeError:
+            sec_grp = self.create_security_group()
         security_groups = [
             {'name': sec_grp['name']}
         ]
@@ -1257,7 +1347,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             # Wait until kuryr-controller pools are reloaded, i.e.,
             # kuryr-controller is ready
             res = test_utils.call_until_true(
-                self.get_pod_readiness, 30, 1, kuryr_pod_name,
+                self.get_pod_readiness, 60, 1, kuryr_pod_name,
                 namespace=system_namespace, container_name='controller')
             self.assertTrue(res, 'Timed out waiting for '
                                  'kuryr-controller to reload pools.')
@@ -1294,6 +1384,7 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
                                             labels=None,
                                             pod_num=None,
                                             pod_name=None,
+                                            service_name=None,
                                             cleanup=True):
         """Verify client pod to service connectivity
 
@@ -1302,11 +1393,12 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
 
         :param service_port - The port of the service we check
         :param protocol - The service protocol we check
-        :namespace - The namespace of the client pod
+        :param namespace - The namespace of the client pod
         :param labels - The labels of the client pod
         :param pod_num - The number of pods expected to serve the service
         :param pod_name - If supplied no pod will be created and instead a pod
                           with this name will be used
+        :param service_name - name of the service to test
         :param cleanup - Whether to add a cleanup function for the created pod
         :returns: The name of the client pod that was created or passed to
                   the function
@@ -1314,12 +1406,19 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
         # FIXME(itzikb): Use the clusterIP to
         # check service status as there are some issues with the FIPs
         # and OVN gates
-        clusterip_svc_ip = self.get_service_ip(self.service_name,
+        service_name = service_name or self.service_name
+        clusterip_svc_ip = self.get_service_ip(service_name,
                                                spec_type='ClusterIP',
                                                namespace=namespace)
         pod_num = pod_num or self.pod_num
         if not pod_name:
-            pod_name, _ = self.create_pod(namespace=namespace, labels=labels)
+            if protocol == "SCTP":
+                pod_name, _ = self.create_pod(
+                    labels=labels, image='quay.io/kuryr/sctp-demo',
+                    namespace=namespace)
+            else:
+                pod_name, _ = self.create_pod(
+                    namespace=namespace, labels=labels)
             if cleanup:
                 self.addClassResourceCleanup(self.delete_pod, pod_name,
                                              namespace=namespace)
@@ -1331,3 +1430,35 @@ class BaseKuryrScenarioTest(manager.NetworkScenarioTest):
             protocol,
             namespace_name=namespace)
         return pod_name
+
+    def get_curl_template(self, ip_or_cidr, extra_args='', port=False):
+        ipn = netaddr.IPNetwork(ip_or_cidr)
+
+        curl_tmpl = "curl " + extra_args if extra_args else "curl"
+        curl_tmpl = (curl_tmpl + " [{}]"
+                     if ipn.version == 6 else curl_tmpl + " {}")
+
+        return curl_tmpl + "{}" if port else curl_tmpl
+
+    def check_updated_listener_timeout(self, service_name,
+                                       namespace='default'):
+        annotation = {
+            'openstack.org/kuryr-timeout-client-data': '70000',
+            'openstack.org/kuryr-timeout-member-data': '75000'
+            }
+        updated_service = self.update_service(service_name=service_name,
+                                              annotation=annotation,
+                                              namespace=namespace)
+        annotated_values = [value for i, value in annotation.items()]
+        start = time.time()
+        while time.time() - start < consts.LB_TIMEOUT:
+            time.sleep(5)
+            timeout_cli, timeout_mem = self.get_listener_timeout_on_crd(
+                service_name=updated_service.metadata.name,
+                namespace=namespace)
+
+            lb_status_values = [str(timeout_cli), str(timeout_mem)]
+            if annotated_values == lb_status_values:
+                break
+
+        self.assertEqual(annotated_values, lb_status_values)
